# -*- coding: utf8 -*-
'''
DB MODULE - generic ODBC
*******************************
***  DB MODULE - generic ODBC
*******************************
$RCSfile: dbmod_odbc.py,v $
$Revision: 996 $
$Author: markus $
$Date: 2010-09-16 11:33:31 +0200 (Do, 16. Sep 2010) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

import os
from string								import join
from biocase.wrapper.errorclasses 		import *
from biocase.tools.various_functions 	import unique, flatten, scalarpNoList
from biocase.wrapper.typeconverter		import DBtransformerClass
from biocase.wrapper.sql.operators		import getWhereObjTables, logicalOperator, compressOperator
from biocase.wrapper.sql.base 			import sqlBase
from biocase.wrapper.sql.select 		import sqlSelect
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod")


# uses inherited ANSI SQL
sqlBase.templates['attribute'] = '"#1"."#2"'
sqlBase.templates['table'] = '"#1"'
sqlBase.templates['alias'] = '#1 #2'
sqlBase.templates['join'] = '#1 JOIN #2 ON #3'
sqlBase.templates['select-limit'] = None
		

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
	__version__ = 'ceODBC'
	__dbmod__	= u'4D ODBC DB module'
	

	# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	def __init__(self, psfObj):
		# setting the psfObj reference to self.psfObj
		# call createDBconnection method
		DBBasemodule.__init__(self, psfObj)
		#
		# read DB 8bit encoding
		#
		#log.debug("8bit DB Encoding: %s"%(psfObj.encoding))
		#self.conn.encoding = psfObj.encoding

	# ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
	# --------------------------------------------------------------------------------
	def createDBconnection(self):
		import ceODBC as odbc			
		loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user ,self.psfObj.password )
		conn = odbc.connect(loginParas)
		return conn


	def fromClause(self, tableList):
		'''Returns the SQL string of the FROM clause for a given table(alias) list.
		Makes use of the tablegraph to produce an ordered list of tables
		and their relation - the relObj (table-alias names, whereObj).'''
		if len(tableList) > 1:
			TreeJoinLists = self.CMFObject.tableTreeObj.getJoinTableListOfLists(tableList)
			AdditionalWhereObjs = []
			fromClause = None
			for t, relObj in flatten(TreeJoinLists, scalarpNoList):
				if relObj is not None:
					#relObj.type = 'ora_left_join'
					AdditionalWhereObjs.append(relObj)
				if fromClause is None:
					fromClause = self.getTableSyntax(t)
				else:
					fromClause += ', '+ self.getTableSyntax(t)
			addWhereObj = compressOperator(logicalOperator('and', AdditionalWhereObjs))
			return (fromClause, addWhereObj)
		else:
			return (self.getTableSyntax(tableList[0]), None)	
