# -*- coding: utf8 -*-
'''
DB MODULE - ODBC for TSQL
*******************************
***  DB MODULE - ODBC for TSQL
*******************************
$RCSfile: dbmod_odbc_tsql.py,v $
$Revision: 1066 $
$Author: markus $
$Date: 2011-06-06 17:22:13 +0200 (Mo, 06. Jun 2011) $

The biocase.wrapper database module to access MS SQL Server databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''


import os
from string                                import join
from datetime                           import datetime
from biocase.wrapper.errorclasses         import *
from biocase.wrapper.typeconverter        import DBtransformerClass
from biocase.wrapper.sql.base             import sqlBase
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod")




#############################################
#
# CUSTOMIZATION for SQL Server
#
# OVERLOAD BASE CLASSES AND OTHER METHODS
#
#############################################

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'



# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
    __version__ = '1.01 using ceODBC ?'
    __dbmod__    = u'MS SQL Server module'
    
    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def __init__(self, psfObj):
        # setting the psfObj reference to self.psfObj
        # call createDBconnection method
        DBBasemodule.__init__(self, psfObj)
       

    # ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
    # --------------------------------------------------------------------------------
    def createDBconnection(self):
        import ceODBC as odbc
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user ,self.psfObj.password )
        conn = odbc.connect( loginParas )        
        return conn 

    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def getTables(self):
        tables = []
        ignore = ["dtproperties", "sysconstraints", "syssegments"]
        for t in self.conn.tables():
            if (str(t[3]) == 'TABLE' or str(t[3]) == 'VIEW') and str(t[1])[0:3] == 'dbo' and not unicode(t[2]) in ignore:
                tables.append(unicode(t[2]))
        return tables
    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def getColumns(self, _table):
        try:
            ret = [unicode(c[3]) for c in self.conn.columns(catalog=None, schema=None, table=_table)]
            if ret == None:
                return []
            return ret
        except:
            log.error("error fetching columns")
            return []
        
    def hasMetaInfo(self):
        return True